SuperStrict

Rem
bbdoc: SQL Database Interface
End Rem
Module Pub.ISQLDB

ModuleInfo "Name: Pub.ISQLDB v1.00"
ModuleInfo "Description: BlitzMax Interface for accessing SQL databases"
ModuleInfo "License: freebie"
ModuleInfo "Authors: gman"
ModuleInfo "History: 2006/1/18 - initial release v1.00"

Import BRL.Blitz

Rem
bbdoc: Sets the current database driver
returns: An instance of ISQLDB or Null if none set
EndRem
Function SetISQLDBDriver:ISQLDB(driver:ISQLDB=Null)
	_isqldbdriver=driver
	Return GetISQLDBDriver()
EndFunction

Rem
bbdoc: Gets the current database driver
returns: An instance of ISQLDB or Null if none set
EndRem
Function GetISQLDBDriver:ISQLDB()
	Return _isqldbdriver
EndFunction

Rem
bbdoc: Interface for an SQL database connection
EndRem
Type ISQLDB Abstract

	Rem
	bbdoc: Status of OK
	EndRem
	Const ERR_OK:Int=0
	Rem
	bbdoc: No information has been set
	EndRem
	Const ERR_NONE:Int=-9999
	Rem
	bbdoc: A general error has occurred
	EndRem
	Const ERR_GENERAL:Int=-9998	
	Rem
	bbdoc: An unknown error has occurred.  Check the error messages for more info.
	EndRem
	Const ERR_UNKNOWN:Int=-9997
	Rem
	bbdoc: The called functionality is not implemented in the underlying engine
	EndRem
	Const ERR_NOT_SUPPORTED:Int=-9996
	Rem
	bbdoc: Initialization of the SQL database library has failed
	EndRem
	Const ERR_INIT_FAIL:Int=-9995
	Rem
	bbdoc: Connection to the specified server could not be made or is no longer valid
	EndRem
	Const ERR_CONNECT_FAIL:Int=-9994
	Rem
	bbdoc: Database server failed to process the sent command
	EndRem
	Const ERR_SRVCMD_FAIL:Int=-9993
	Rem
	bbdoc: Database server failed to process the sent SQL statement
	EndRem	
	Const ERR_QUERY_FAIL:Int=-9992
	Rem
	bbdoc: Indicates a problem with the returned query result handle
	EndRem
	Const ERR_RESULT_FAIL:Int=-9991
	Rem
	bbdoc: Indicates that no results were returned
	EndRem
	Const ERR_NO_RESULTS:Int=-9990
	Rem
	bbdoc: An invalid column name or index has been referenced
	EndRem	
	Const ERR_INVALID_COLUMN:Int=-9989
	Rem
	bbdoc: An invalid row and column has been referenced
	EndRem	
	Const ERR_INVALID_ROWCOL:Int=-9988
						
	Rem
	bbdoc: Gets the database server being accessed
	EndRem
	Method getHost:String() Abstract
	Rem
	bbdoc: Gets the database that the server is accessing
	EndRem
	Method getDB:String() Abstract
	Rem
	bbdoc: Gets the username used to login to the server
	EndRem
	Method getUsername:String() Abstract
	Rem
	bbdoc: Gets the password used to login to the server
	EndRem	
	Method getPassword:String() Abstract
	Rem
	bbdoc: Gets the port used to connect to the server
	EndRem		
	Method getPort:Int() Abstract
	
	Rem
	bbdoc: Sets the event handler that will fire each time a result set is received in its entirety
	about: This is global to the type, not the instance
	EndRem		
	Function setOnEndResultHandler(handler:IOnEndResultEventHandler=Null) Abstract

	Rem
	bbdoc: Sets the event handler that will fire each time a result set is received
	about: This is global to the type, not the instance.  This may not be implemented for some engines.
	EndRem		
	Function setOnResultHandler(handler:IOnResultEventHandler=Null) Abstract
															
	Rem
	bbdoc: Gets the native handle For this instance of ISQLDB
	about: "Native" refers To the underlying SQL database engine connection handle.  This handle can be used To access native engine functions directly.
	EndRem
	Method getNativeHandle:Int() Abstract
	Rem
	bbdoc: Gets the error type of the last ISQLDB error that occurred
	EndRem		
	Method getError:Int() Abstract
	Rem
	bbdoc: Gets the error message of the last ISQLDB error that occurred
	EndRem		
	Method getErrorMsg:String() Abstract
	Rem
	bbdoc: Gets the native error message of the last error that occurred
	about: "Native" refers to the error as reported by the underlying SQL database engine
	EndRem		
	Method getNativeError:String() Abstract	
	Rem
	bbdoc: Attempts to open a connection to the server
	returns: True if a successful connection was made, otherwise False
	EndRem		
	Method openConnection:Int() Abstract
	Rem
	bbdoc: Closes an open a connection to the server
	EndRem			
	Method closeConnection() Abstract
	
	Rem
	bbdoc: Executes an SQL query on the connected database server
	returns: An instance of ISQLResult or Null if failed
	about: Used for SELECT SQL statements.  If the return value is Null, check the error information on ISQLDB.
	EndRem
	Method execute:ISQLResult(query:String) Abstract
	
	Rem
	bbdoc: Executes an SQL statement on the connected database server that does not return data as a result
	returns: An instance if ISQLResult or Null if failed
	about: Used for INSERT, UPDATE, and DELETE SQL statements.  While the result will not contain rows, getNumRows() will tell the number of rows affected and getInsertId() will tell the autocreated record Id if there was one (and the implementation supports it).
	EndRem
	Method executeNonQuery:ISQLResult(stmt:String) Abstract
	
	Rem
	bbdoc: Executes an SQL query on the connected database server and returns a single value from the first row of the result set
	returns: If field_nm parameter is used, the value of the passed field in the first row of the result set, else the value of the first row, first column of the result set.
	about: Returned value can be Null.  Check error information to see if an error occurred.  Error will be ERR_NO_RESULTS if the query resulted in no returned rows.
	EndRem
	Method executeScalar:String(query:String,field_nm:String=Null) Abstract
	
EndType

Rem
bbdoc: For implementations that can keep data on the server and fetch incrementally, an instance of this can be set and it will fire for each row read
EndRem
Type IOnResultEventHandler Abstract
	Method OnResult(result:ISQLResult,row:Int) Abstract
EndType

Rem
bbdoc: Set an instance of this on an implementation to know when a result has been completely processed
about: Every implementation should have this ability but for some implementations it may not fire at the same time the ISQLResult is actually returned.  This is due to some impelentations having the ability to keep data on the actual server until that row is requested.
EndRem
Type IOnEndResultEventHandler Abstract
	Method OnEndResult(result:ISQLResult) Abstract
EndType

Rem
bbdoc: The result of an ISQLDB query providing access to the returned data
EndRem
Type ISQLResult Abstract
	Rem
	bbdoc: Gets the instance of ISQLDB that created the result
	EndRem
	Method getConnection:ISQLDB() Abstract
	Rem
	bbdoc: Gets the SQL query statement that created this result
	EndRem
	Method getQuery:String() Abstract
	
	Rem
	bbdoc: Gets the error type of the last ISQLDB error that occurred
	EndRem		
	Method getError:Int() Abstract
	Rem
	bbdoc: Gets the error message of the last ISQLDB error that occurred
	EndRem		
	Method getErrorMsg:String() Abstract
	Rem
	bbdoc: Gets the native error message of the last error that occurred
	about: "Native" refers to the error as reported by the underlying SQL database engine
	EndRem		
	Method getNativeError:String() Abstract	
	
	Rem
	bbdoc: Gets if the query was successful or not
	returns: True/False
	about: Check error information for details on why the query failed
	EndRem	
	Method getSuccess:Int() Abstract
	Rem
	bbdoc: Gets whether all the results have been returned
	returns: True/False
	EndRem
	Method getComplete:Int() Abstract ' returns true if all results have been downloaded
	Rem
	bbdoc: Gets the number of rows in this result or the # of rows affected by the SQL statement
	EndRem
	Method getNumRows:Int() Abstract
	Rem
	bbdoc: Gets the number of columns in this result
	EndRem
	Method getNumCols:Int() Abstract
	Rem
	bbdoc: Gets the column name of the passed index
	returns: The column name or Null if not found
	EndRem
	Method getColName:String(col:Int) Abstract
	Rem
	bbdoc: Gets the index of the passed column name
	returns: The index of the column or ERR_INVALID_COLUMN if not found
	EndRem	
	Method getColIdx:Int(col:String) Abstract
	Rem
	bbdoc: Gets the width in bytes of the data contained in a row/column
	EndRem
	Method getColWidth:Int(row:Int,col:Int) Abstract
	Rem
	bbdoc: Gets the value of a column by row and column index
	returns: The column value as a string
	about: Can check for ERR_INVALID_COLUMN or ERR_INVALID_ROWCOL errors to see if this failed
	EndRem	
	Method getVal:String(row:Int,col:Int) Abstract
	Rem
	bbdoc: Gets the value of a column by row index and column name
	returns: The column value as a string
	about: Can check for ERR_INVALID_COLUMN or ERR_INVALID_ROWCOL errors to see if this failed
	EndRem
	Method getValByName:String(row:Int,col:String) Abstract
	Rem
	bbdoc: Gets the value of a column by row and column index
	returns: The column value as an Integer
	about: A RuntimeError will be generated if an invalid row or column is passed
	EndRem	
	Method getIntVal:Int(row:Int,col:Int) Abstract
	Rem
	bbdoc: Gets the value of a column by row index and column name
	returns: The column value as an Integer
	about: A RuntimeError will be generated if an invalid row or column is passed
	EndRem	
	Method getIntValByName:Int(row:Int,col:String) Abstract
	Rem
	bbdoc: Gets the value of a column by row and column index
	returns: The column value as a Long
	about: A RuntimeError will be generated if an invalid row or column is passed
	EndRem	
	Method getLongVal:Long(row:Int,col:Int) Abstract
	Rem
	bbdoc: Gets the value of a column by row index And column name
	returns: The column value as a Long
	about: A RuntimeError will be generated If an invalid row or column is passed
	EndRem	
	Method getLongValByName:Long(row:Int,col:String) Abstract
	Rem
	bbdoc: Gets the value of a column by row and column index
	returns: The column value as a Float
	about: A RuntimeError will be generated if an invalid row or column is passed
	EndRem	
	Method getFloatVal:Float(row:Int,col:Int) Abstract
	Rem
	bbdoc: Gets the value of a column by row index And column name
	returns: The column value as a Float
	about: A RuntimeError will be generated If an invalid row or column is passed
	EndRem	
	Method getFloatValByName:Float(row:Int,col:String) Abstract
	Rem
	bbdoc: Gets the value of a column by row and column index
	returns: The column value as a Double
	about: A RuntimeError will be generated if an invalid row or column is passed
	EndRem	
	Method getDoubleVal:Float(row:Int,col:Int) Abstract
	Rem
	bbdoc: Gets the value of a column by row index And column name
	returns: The column value as a Double
	about: A RuntimeError will be generated If an invalid row or column is passed
	EndRem	
	Method getDoubleValByName:Float(row:Int,col:String) Abstract

	Rem
	bbdoc: Gets the last Id generated for a new record by the SQL statement that generated this result
	returns: The new Id as a String
	EndRem	
	Method getInsertId:String() Abstract
	
	Rem
	bbdoc: Gets the native handle for this instance of ISQLResult
	about: "Native" refers to the underlying SQL database engine result handle.  This handle can be used to access native engine functions directly.
	EndRem
	Method getNativeHandle:Int() Abstract
EndType

Private

Rem
bbdoc: Private variable storing a global reference to the current ISQLDB driver
EndRem
Global _isqldbdriver:ISQLDB=Null
